#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <windows.h>
///
/// www.boost.org
///
#include <boost/heap/fibonacci_heap.hpp>

using namespace std;

#define MAX_N 300
#define TIME_LIMIT (1900LL)

typedef unsigned long long int bigint_t;

static unsigned int A_inst[MAX_N][MAX_N];

static SYSTEMTIME start_time;
bigint_t start_time_ms;
void startTime()
{
	FILETIME ft_now;
	GetSystemTimeAsFileTime(&ft_now);
	start_time_ms = (LONGLONG)ft_now.dwLowDateTime + ((LONGLONG)(ft_now.dwHighDateTime) << 32LL);
	start_time_ms /= 10000;
}
int timesUp()
{
	FILETIME ft_now;
	bigint_t now_ms;
	//GetSystemTime(&start_time);
	GetSystemTimeAsFileTime(&ft_now);
	now_ms = (LONGLONG)ft_now.dwLowDateTime + ((LONGLONG)(ft_now.dwHighDateTime) << 32LL);
	now_ms /= 10000;
	if ((now_ms-start_time_ms) >= TIME_LIMIT) {
		return 1;
	}
	return 0;
}

struct link_t {
	int from;
	int to;
	int len;
	int pos;
	int used;
	bigint_t travel_cnt;
	int key;
	bigint_t gain;
	link_t() : from(0), to(0), len(0), pos(0), used(0), travel_cnt(0), key(0), gain(0) {
		
	}
};

static link_t *links_ptr[MAX_N*MAX_N];

class DJSet {
public:
	DJSet();
	~DJSet();
	void MakeSet();
	DJSet * Find();
	void Union(DJSet *djs2);

private:
	DJSet *parent;
	int rank;
};

DJSet::DJSet()
{

}
DJSet::~DJSet()
{

}

void DJSet::MakeSet()
{
	DJSet *djs = this;

	djs->parent = djs;
	djs->rank = 0;
}

DJSet * DJSet::Find()
{
	DJSet *djs = this;
	if (djs->parent != djs) {
		djs->parent = djs->parent->Find();
	}
	return djs->parent;
}

void DJSet::Union(DJSet *djs2)
{
	DJSet *djs1 = this;
	if (djs1->rank < djs2->rank) {
		djs2->parent = djs1;
	} else if (djs2->rank < djs1->rank) {
		djs1->parent = djs2;
	} else {
		djs2->parent = djs1;
		djs1->rank++;
	}
}


class Node : public DJSet {
public:
	Node(int idx=0);
	int	Idx();
	void link(Node *n, link_t *link);
	void unlink(Node *n);
	bool operator<(Node &n) {
		return dist < n.dist;
	}
	void link(Node *n, int d);
	void link_tree(Node *n, link_t *link);
private:
	void link2(Node *n, link_t *link);
	void unlink2(Node *n);
public:
	struct le {
		le *next;
		int dist;
		link_t *link;
		Node *node;
		//le(Node *n, le *next, int d) :node(n), next(next), dist(d){}
	};
	le *port;
	le *tree;
	int idx;
	int dist;
	bigint_t visit_total;
	int visited;
};

Node::le node_links[2*MAX_N*MAX_N];
Node::le *node_links_stack[2*MAX_N*MAX_N];
static int links_idx = 0;
void init_links()
{
	for (int i = 0; i < 2*MAX_N*MAX_N; i++) {
		node_links_stack[i] = &node_links[i];
	}
}
Node::le *new_link(Node *n, Node::le *next, link_t *link)
{
	Node::le *lle = node_links_stack[links_idx++];
	lle->link = link;
	lle->next = next;
	lle->node = n;

	return lle;
}
void del_link(Node::le *lle)
{
	node_links_stack[--links_idx] = lle;
}

struct prio_el {
	Node *node;
	unsigned int dist;
	bool operator<(prio_el const &el) const { return dist > el.dist; }
	prio_el(Node *n, int d) : node(n), dist(d){}
};
typedef boost::heap::fibonacci_heap<prio_el> PrioQ;


Node::Node(int idx)
{
	this->idx = idx;
	port = NULL;
	tree = NULL;
	visited = 0;
}

int Node::Idx()
{
	return idx;
}

void Node::link(Node *n, int d)
{

}

void Node::link_tree(Node *n, link_t *link)
{
	le *nle = new_link(n, tree, link); //new le(n, port, d);
	tree = nle;
}
void Node::link(Node *n, link_t *link)
{
	link2(n, link);
	n->link2(this, link);
}
void Node::link2(Node *n, link_t *link)
{
	le *nle = new_link(n, port, link); //new le(n, port, d);
	port = nle;
}

void Node::unlink(Node *n)
{
	unlink2(n);
	n->unlink2(this);
}
void Node::unlink2(Node *n)
{
	le *lle = port;
	if (port->node == n) {
		port = port->next;
		del_link(lle);//delete lle;
		return;
	}
	
	while (lle->next && (lle->next->node != n))
		lle = lle->next;
	if (lle->next) {
		le *cur = lle->next;
		lle->next = lle->next->next;
		del_link(cur);
	}
}



static Node nodes[MAX_N];
static Node *nodes_sort[MAX_N];

class GraphSolver {
public:
	GraphSolver(int M, int N, unsigned int (*A)[MAX_N][MAX_N], Node *nodes, link_t *links);
	~GraphSolver();
	int * Solve(int *);
private:
	 int M, N;
	 unsigned int (*A)[MAX_N][MAX_N];
	 Node *nodes;
	 link_t *links;
	 bigint_t best_total;
	 bigint_t cur_total;
	 unsigned int dist_a[MAX_N][MAX_N];
	 int current_res[MAX_N];
	 int final_res[MAX_N];
	 int res_idx;
	 int prev[MAX_N];
	 int prev_old[MAX_N];
	 bigint_t root_total_dist;
	 int traveled[MAX_N];
private:
	 void build_graph(void);
	 
	 static int comp2(const void * ll1, const void * ll2);
	 static int comp_nodes(const void * n1, const void * n2);
	 static int comp_link(const void *ll1, const void *ll2);
	 unsigned int & dist(int f, int t);
	 void updateLinks(int root);
	 int dji(int root);
};

GraphSolver::GraphSolver(int M, int N, unsigned int (*A)[MAX_N][MAX_N], Node *nodes, link_t *links) :
	best_total(-1), cur_total(0), root_total_dist(0)
{
	this->M = M;
	this->N = N;
	this->A = A;
	this->nodes = nodes;
	this->links = links;
	for (int i = 0; i < N; i++)
		for (int j = 0; j < N; j++)
			dist_a[i][j] = 0;
	memset(traveled,0x0, sizeof(traveled));
}

GraphSolver::~GraphSolver()
{
}


 int GraphSolver::comp2(const void * l1, const void * l2)
{
	int *ll1 = (int *) l1;
	int *ll2 = (int *) l2;

	if (*ll1 < *ll2) {
		return -1;
	} else if (*ll1 > *ll2) {
		return 1;
	} else {
		return 0;
	}
}

 int  GraphSolver::comp_nodes(const void * n1, const void * n2)
 {
	Node **nn1 = (Node **) n1;
	Node **nn2 = (Node **) n2;

	if ((*nn1)->visit_total > (*nn2)->visit_total) {
		return -1;
	} else if ((*nn1)->visit_total < (*nn2)->visit_total) {
		return 1;
	} else {
		return 0;
	}
 }

 int GraphSolver::comp_link(const void *ll1, const void *ll2)
{
	link_t *l1 = (link_t *) ll1;
	link_t *l2 = (link_t *) ll2;

	if (l1->travel_cnt > l2->travel_cnt) {
		return -1;
	} else if (l1->travel_cnt < l2->travel_cnt) {
		return 1;
	} else {
		return 0;
	}
}
 
 unsigned int & GraphSolver::dist(int f, int t)
 {
	 return (f < t) ? dist_a[f][t] : dist_a[t][f];
 }


void GraphSolver::build_graph()
{
	for (int i = 0; i < M; i++) {
		link_t *l = &links[i];
		nodes[l->from].link(&nodes[l->to], l);
	}
}


void GraphSolver::updateLinks(int root)
{
	int to;

	for (int i = 0; i < N; i++) {
		
		if (i == root)
			continue;
		to = i;
		do {
			link_t *l = &links[prev[to]];
			l->travel_cnt += A_inst[root][i];
			root_total_dist += (bigint_t)l->len * A_inst[root][i];
			to = (l->to == to) ? l->from : l->to;
		} while (root != to);
	}
}

int GraphSolver::dji(int j)
{
	int nods = 0;
	PrioQ priQ;
	for (int i = 0; i < N; i++) {
		nodes[i].visited = 0;
		for (int j = i; j < N; j++) {
			dist(i, j) = 0xFFFFFFFF;
		}
	}

	int root = nodes_sort[j]->idx;
	
	dist(root, root) = 0;

	prev[root] = 999999;
	prio_el priel = prio_el(nodes_sort[j], dist(root, root));
	priQ.push(priel);

	while (priQ.size()) {
		prio_el priel = priQ.top();
		priQ.pop();
		Node *n = priel.node;
		if (n->visited)
			continue;
		for (Node::le *lle = n->port; lle; lle = lle->next) {
			Node * nd = lle->node;
			if (nd != n) {
				if (!nd->visited && ((dist(root, n->idx) + lle->link->len) < dist(root, nd->idx))) {
					dist(root, nd->idx) = (dist(root, n->idx) + lle->link->len);
					prio_el priel = prio_el(nd, dist(root, nd->idx));
					priQ.push(priel);
					prev[nd->idx] = lle->link->pos;
				}
			}
		}
		n->visited = 1;
		nods++;
	}
	return nods;
}
int * GraphSolver::Solve(int *res)
{
	int *orig_res = res;
	init_links();
	build_graph();
	
	qsort(nodes_sort, N, sizeof(Node*), comp_nodes);
	
	for (int j = 0; j < N; j++) {
		
		dji(j);
		
		updateLinks(nodes_sort[j]->idx);
		
		if (timesUp())
			break;
	}
	qsort(links, M, sizeof(link_t), &GraphSolver::comp_link);

	for (int i = 0; i < N; i++) {
		nodes[i].MakeSet();
	}
	
	int sets = N;
	for (link_t *link = links; sets > 1; link++) {
		DJSet *set1, *set2;
		Node *from = &nodes[link->from];
		Node *to = &nodes[link->to];

		set1 = from->Find();
		set2 = to->Find();
		if (set1 == set2) {
			continue;
		}
		link->used = 1;
		set1->Union(set2);
		sets--;
		*res++ = link->pos;
	}
	
	qsort(orig_res, N-1,sizeof(int),comp2);
	orig_res[N-1] = -1;
	
	return NULL;
}

#define SWAP(a,b) ((a) ^= (b),(b) ^= (a), (a) ^= (b))

int main(int argc, char **argv)
{
	int N, M;
	startTime();

	FILE *inf;
	inf = fopen("graph.in", "r");
	if (NULL == inf) {
		perror("Error opening infile");
		return 1;
	}
	FILE *outf;
	outf = fopen("graph.out", "w");
	if (NULL == inf) {
		perror("Error opening outfile");
		return 1;
	}
	fscanf(inf, "%d %d", &N, &M);

	link_t *links = new link_t[M+1];
	int i;
	for (i = 0; i < M; i++) {
		fscanf(inf, "%d %d %d", &links[i].from, &links[i].to, &links[i].len);
		if (links[i].from > links[i].to) {
			SWAP(links[i].from, links[i].to);
		}
		links[i].pos = i;
		links[i].from--;
		links[i].to--;
		links[i].pos;
		
	}
	links[i].len = -1;
	for (int i = 0; i < N; i++) {
		nodes[i] = Node(i);
		nodes_sort[i] = &nodes[i];
	}
	
	for (int i = 0; i < N; i++) {
		bigint_t visit_cnt = 0;
		for (int j = 0; j < N; j++) {
			fscanf(inf, "%d", &A_inst[i][j]);
			visit_cnt += A_inst[i][j];
		}
		nodes[i].visit_total = (bigint_t) visit_cnt;
	}

	
	int *res = new int[N];
	res[N-1] = -1;
	GraphSolver *solver = new GraphSolver(M, N, &A_inst, &nodes[0], &links[0]);

	solver->Solve(res);
	for (int *sol = res; *sol >= 0; sol++) {
		fprintf(outf, "%d\n", *sol + 1);
	}

	delete solver;
	delete links;
	delete res;
	fclose(inf);
	fclose(outf);
}